from django.contrib import admin
from django.http.request import HttpRequest
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_protect

csrf_protect_m = method_decorator(csrf_protect)


class PreferencesUtilsAdmin(admin.ModelAdmin):
    # override to change submit row
    change_form_template = "admin/preferences_utils/change_form.html"

    @csrf_protect_m
    def changelist_view(self, request: HttpRequest, extra_context=None):
        """
        Display single preferences object as a readonly changeform
        """
        model = self.model
        obj = model.instance()
        return redirect(
            "%s?readonly=1" # readonly to have a view object
            % reverse(
                "admin:%s_%s_change" % (model._meta.app_label, model._meta.model_name),
                args=(obj.id,),
            )
        )

    def has_change_permission(self, request, obj=None):
        return request.GET.get("readonly") != "1"

    def has_add_permission(self, request: HttpRequest):
        return False



# -*- coding: utf-8 -*-
import importlib

"""
test_django-preferences-utils
------------

Tests for `django-preferences-utils` context processors
"""

from django.test import TestCase, RequestFactory

class PreferencesUtilsContextProcessorTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_context(self):
        request = self.factory.get('/')
        cls = importlib.import_module("preferences_utils.context_processors.preferences")
        pref = cls.pref(request).get('pref')
        self.assertEqual(pref.site_title, "Site Title")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
test_django-preferences-utils
------------

Tests for `django-preferences-utils` templatetags module.
"""

from django.test import TestCase
from preferences_utils.templatetags.preferences_utils import get_dict_item

class PreferencesUtilsModelTagsTest(TestCase):
    def test_get_dict_key(self):
        mydict = {
            'key1': 'value1',
            'key2': 'value2',
        }
        self.assertEqual(get_dict_item(mydict, 'key1'), 'value1')
        self.assertEqual(get_dict_item(mydict, 'key2'), 'value2')
        self.assertEqual(get_dict_item(mydict, 'key3'), None)



import sys
from .models import PreferencesUtilsModel

class PrefProcessor(object):
    def __getattr__(self, attr):
        if attr == '__file__':
            # autoreload support in dev server
            return __file__
        else:
            return lambda request: {attr: PreferencesUtilsModel.__subclasses__[attr].instance()}

sys.modules[__name__ + '.preferences'] = PrefProcessor()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
test_django-preferences-utils
------------

Tests for `django-preferences-utils` admin module.
"""

from django.test import TestCase, RequestFactory

from django.contrib.admin.sites import AdminSite
from django.views.generic.base import HttpResponseRedirect
from tests.admin import PreferencesAdmin
from tests.models import Preferences


class PreferencesUtilsAdminTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_changelist_view(self):
        request = self.factory.get('/admin/tests/preferences/')
        admin = PreferencesAdmin(model=Preferences, admin_site=AdminSite())
        res: HttpResponseRedirect = admin.changelist_view(request)
        self.assertEqual(res.status_code, 302)
        self.assertEqual(res.url, '/admin/tests/preferences/1/change/?readonly=1')

    def test_has_change_permission_readonly(self):
        request = self.factory.get('/admin/tests/preferences/1/change/?readonly=1')
        admin = PreferencesAdmin(model=Preferences, admin_site=AdminSite())
        self.assertEqual(admin.has_change_permission(request), False)

    def test_has_change_permission_not_readonly(self):
        request = self.factory.get('/admin/tests/preferences/1/change/')
        admin = PreferencesAdmin(model=Preferences, admin_site=AdminSite())
        self.assertEqual(admin.has_change_permission(request), True)

    def test_has_add_permission(self):
        request = self.factory.get('/admin/tests/preferences/')
        admin = PreferencesAdmin(model=Preferences, admin_site=AdminSite())
        self.assertEqual(admin.has_add_permission(request), False)



from django.db import models


class PreferencesUtilsModel(models.Model):
    __subclasses__ = {}

    @classmethod
    def register(cls, name):
        """ Decorator to register a subclass 
            We can have multiple subclasses, so we keeps track of them
        """
        if cls.__name__ != "PreferencesUtilsModel":
            raise Exception("You can only register subclasses of PreferencesUtilsModel")

        def wrapper(subcls):
            cls.__subclasses__[name] = subcls
            return subcls

        return wrapper

    @classmethod
    def instance(cls, name=""):
        """
        If called from a specific subclass name is not required,
        if called from the base class, the name is required
        """
        if name == "" and not hasattr(cls, "objects"):
            raise Exception(
                "You must specify the subclass name if called from the base class"
            )

        kls = cls if name == "" else cls.__subclasses__[name]
        item = kls.objects.first()
        if not item:
            item = kls(id=1)
            item.save()
        return item

    def save(self, *args, **kwargs):
        """ Assure to save only one instance """
        self.id = 1
        super().save(*args, **kwargs)

    def __str__(self):
        return self.__class__.__name__

    class Meta:
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
test_django-preferences-utils
------------

Tests for `django-preferences-utils` models module.
"""

from django.test import TransactionTestCase

from preferences_utils.models import PreferencesUtilsModel
from .models import Preferences, Preferences2


class PreferencesUtilsModelTest(TransactionTestCase):
    def test_creation(self):
        item = Preferences(
            site_title='Meow',
        )
        item.save()
        item2 = Preferences(
            site_title='Meow',
        )
        item2.save()
        self.assertEqual(item2.id, 1)
        self.assertEqual(str(item2), 'Preferences')

    def test_get_instance_from_derived(self):
        item = Preferences(
            site_title='Meow',
        )
        item.save()
        instance = Preferences.instance()
        self.assertEqual(instance.id, 1)
        self.assertEqual(instance.site_title, 'Meow')

    def test_get_instance_from_base(self):
        item = Preferences(
            site_title='Meow',
        )
        item.save()
        item2 = Preferences2(
            site_title2='Meow',
        )
        item2.save()
        instance = PreferencesUtilsModel.instance('pref')
        instance2 = PreferencesUtilsModel.instance('pref2')
        self.assertEqual(instance.id, 1)
        self.assertEqual(instance.site_title, 'Meow')
        self.assertEqual(instance2.id, 1)
        self.assertEqual(instance2.site_title2, 'Meow')

    def test_get_instance_from_base_without_name(self):
        item = Preferences(
            site_title='Meow',
        )
        item.save()
        try:
            instance = PreferencesUtilsModel.instance()
            self.fail()
        except Exception as e:
            self.assertRaises(Exception)
